// This is a roxen module. (c) Martin Baehr 1999

// The dirlist module. 
// lists a directory to create a menu from it.

//  This code is (c) 1999 Martin Baehr, and can be used, modified and
//  redistributed freely under the terms of the GNU General Public License,
//  version 2.
//  This code comes on a AS-IS basis, with NO WARRANTY OF ANY KIND, either
//  implicit or explicit. Use at your own risk.
//  You can modify this code as you wish, but in this case please
//  - state that you changed the code in the modified version
//  - do not remove my name from it
//  - send me a copy of the modified version or a patch, so that
//    I can include it in the 'official' release.
//  If you find this code useful, please e-mail me. It would definitely
//  boost my ego :)
//  
//  For risks and side-effects please read the code or ask your local 
//  unix or roxen-guru.

constant cvs_version="$Id: dirlist.pike,v 1.4 1999/09/26 20:11:33 mbaehr Exp $";

#include <module.h>
inherit "module";
inherit "roxenlib";

static private string doc()
{
  return "Defines the <tt>&lt;dirlist&gt;&lt;/dirlist&gt;</tt> container<p>\n"
         "based on an idea by per hedbor, this takes a list of file and "
         "directory names and formats each according to the markup in the "
         "<tt>&lt;dirlist&gt;&lt;/dirlist&gt;</tt> container.<p>\n"
         "The container takes the following attributes:<br>\n"
         "<dl>"
         "<dt><b>dir</b>=<tt>relative directory</tt> (optional)<br>\n"
         "<dd>specifies the directory to be listed instead of the current\n" 
         "<dt><b>quote</b>=<tt>string</tt> (optional)<br>\n"
         "<dd>specifies the string to be used for quoting variables "
         "instead of #\n"
         "<dt><b>dirlist</b>=<tt>comma-sepearated list of filenames</tt> "
         "(optional)<br>\n"
         "<dd>create the listing from this list, instead of listing some dir\n" 
         "<dt><b>namelist</b>=<tt>comma-sepearated list of names</tt> "
         "(optional, requires dirlist arg.)<br>\n"
         "<dd>use the names from this list, instead of creating them from "
	 "the dir listing\n" 
         "</dl><p>\n"
         "in the contents of the container the variables #file# and #name# "
         "(# may be replaced with a different quore-string (see above)) "
         "will be replaced with the filename and the basename of each entry "
         "(the basename here is the part of a filename before the first '.')"
         "<p>\n"
         "<tt>&lt;dirlist&gt;&lt;/dirlist&gt;</tt> "
         "knows the following subcontainer: "
         "<dl>\n"
         "<dt><b><tt>&lt;selected&gt;&lt;/selected&gt;</tt></b> (optional)\n"
         "<dd><tt>&lt;selected&gt;&lt;/selected&gt;</tt> "
         "takes the following attributes:<br>\n"
         "  <dd>\n "
         "  <dl>"
         "  <dt><b>file</b>=<tt>filename</tt> (optional)<br>\n"
         "  <dt><b>name</b>=<tt>name</tt> (optional)<br>\n"
         "  <dd>if the filename of the list-entry matches <b>file</b> "
         "  or the basename matches <b>name</b> "
         "  then it will be formated with the contents of this container\n" 
         "  </dl>\n"
         "</dl>\n";

}

array register_module()
{
  return ({ MODULE_PARSER, "dirlist",
            doc()
            ,0,1 });
}

void create()
{
  defvar("extuni",
         ({ "de", "en", "se" }),
         "unify extensions",
         TYPE_STRING_LIST,
         "treat files with these extensions as one");

  defvar("fignore",
         ({ "index" }),
         "ignore files",
         TYPE_STRING_LIST,
         "ignore files with these names and any extension");

  defvar("extignore",
         ({ "gif", "jpg", "png" }),
         "ignore extensions",
         TYPE_STRING_LIST,
         "ignore files with these extensions");
}

string do_replace(mapping selected, string contents, string file, string name, string quote)
{
  string out;
  //werror("dirlist: replace: %O, %s, %s, %s\n", selected,contents,file,name);
  //werror("dirlist: replace: %s\n", out);

  if(selected[file])
    return replace(selected[file], ({ quote+"file"+quote, quote+"name"+quote }), ({ file, replace(name, "_", " ") }));

  if(selected[name])
    return replace(selected[name], ({ quote+"file"+quote, quote+"name"+quote }), ({ file, replace(name, "_", " ") }));
  
  return replace(contents, ({ quote+"file"+quote, quote+"name"+quote }), ({ file, replace(name, "_", " ") }));
}

string icontainer_data(string container,
                         mapping arguments,
                         string contents,
                         mapping out)
{
  out[arguments->name]=contents;
  out[arguments->file]=contents;
  return "";
}

string container_dirlist(string name, 
                     mapping arguments, 
                     string contents, 
                     object id, 
                     mapping defines) 
{
  if(arguments->help)
    return("<obox><title>&lt;"+name+"&gt;&lt;/"+name+"&gt;</title>"+doc()+"</obox>");

  multiset extuni=mkmultiset(query("extuni"));
  multiset extignore=mkmultiset(query("extignore"));
  multiset fignore=mkmultiset(query("fignore"));
  mapping tmp = ([]);
  mapping data=([]);
  string out="";
  string quote="#";
  string dir=(arguments->dir?arguments->dir:"");
  array dirlist=({}), namelist=({});
  
  if(arguments->quote)
    quote=arguments->quote;

  //werror(sprintf("dirlist: main: %O, %O\n", arguments->dir, id->conf->find_dir(fix_relative(dir, id), id)));

  contents = parse_html(contents, ([]), ([ "selected":icontainer_data ]), data);

  if(arguments->dirlist)
  {
    dirlist=arguments->dirlist/",";
    int i;

    if(arguments->namelist)
    {
      namelist=arguments->namelist/",";
    }
    else
    {
      for(i=0; i<sizeof(dirlist); i++)
      {
        array(string) a=dirlist[i]/".";
        namelist+=({a[0]}); 
      }
    }

    for(i=0; i<sizeof(namelist); i++)
    {
      //werror(sprintf("dirlist: %s:%s\n", dirlist[i], namelist[i]));
      out += do_replace(data, contents, dirlist[i], namelist[i], quote);
    }
  }
  else
  {
    //werror("dirlist: array: %s, %s, %O\n", dir, fix_relative(dir, id), id->conf->find_dir( fix_relative(dir+"/", id), id ));
    if(id->conf->find_dir( fix_relative(dir+"/", id), id ))
    {
      dirlist = sort(Array.filter(id->conf->find_dir( fix_relative(dir+"/", id), id ), 
                    lambda(string s, multiset m, multiset n, multiset o) 
                    { 
                      array(string) a=s/".";
                      return !(m[a[-1]] || n[a[0]] || (o[a[-1]] && a[sizeof(a)-2] && m[a[sizeof(a)-2]])); 
                    } ,               
                    extignore, fignore, extuni)
		   );
    }
    else
      return "can't find directory: "+dir;

    foreach(dirlist, string s)
    {
      array(string) a=s/".";
      if(extuni[a[-1]]) 
        s=a[..sizeof(a)-2]*".";
      //werror(sprintf("dirlist: %s\n", s));
      tmp[s]=a[0]; 
    }
  
    foreach( sort(indices(tmp)), string s)
    {
      //werror(sprintf("dirlist: %s\n", s));
      out += do_replace(data, contents, s, tmp[s], quote);
    }
  }
  return out;
}

mapping query_container_callers()
{
  return ([ "dirlist":container_dirlist, ]);
}
